@@ -10,5 +10,6 @@ from .parameters import action_flag
 
 @action_flag('--help', '-h', order=float('-inf'), help='Show this help message and exit')
 def help_action(self):
-    print(self.params.formatter.format_help())
+    cls = self.__class__
+    print(cls.__class__.params(cls).formatter.format_help())
     raise ParserExit
