@@ -28,7 +28,7 @@ class Command(ABC, metaclass=CommandMeta):
 
     def __new__(cls):
         ctx = _get_or_create_context(cls)
-        # By storing the Context here instead of __init__, every single sub class won't need to
+        # By storing the Context here instead of __init__, every single subclass won't need to
         # call super().__init__(...) from their own __init__ for this step
         self = super().__new__(cls)
         self.__ctx = ctx
@@ -39,17 +39,16 @@ class Command(ABC, metaclass=CommandMeta):
     def parse_and_run(cls, argv: Sequence[str] = None, *args, **kwargs) -> Optional[CommandObj]:
         """
         Primary entry point for parsing arguments, resolving sub-commands, and running a command.  Calls :meth:`.parse`
-        to parse arguments and resolve sub-commands, then calls :meth:`.run` on the resulting Command instance.  Handles
-        exceptions during parsing using the configured :class:`ErrorHandler
-        <command_parser.error_handling.ErrorHandler>`.
+        to parse arguments and resolve sub-commands, then calls :meth:`.__call__` on the resulting Command instance.
+        Handles exceptions during parsing using the configured :class:`.ErrorHandler`.
 
         To be able to store a reference to the (possibly resolved sub-command) command instance, you should instead use
-        the above mentioned methods separately.
+        the above-mentioned methods separately.
 
         :param argv: The arguments to parse (defaults to :data:`sys.argv`)
-        :param args: Positional arguments to pass to :meth:`.run`
-        :param kwargs: Keyword arguments to pass to :meth:`.run`
-        :return: The Command instance with parsed arguments for which :meth:`.run` was already called.
+        :param args: Positional arguments to pass to :meth:`.__call__`
+        :param kwargs: Keyword arguments to pass to :meth:`.__call__`
+        :return: The Command instance with parsed arguments for which :meth:`.__call__` was already called.
         """
         ctx = _get_or_create_context(cls, argv)
         with ctx.get_error_handler():
@@ -70,7 +69,7 @@ class Command(ABC, metaclass=CommandMeta):
         parsed arguments, if necessary.
 
         :param argv: The arguments to parse (defaults to :data:`sys.argv`)
-        :return: A Command instance with parsed arguments that is ready for :meth:`.run` or :meth:`.main`
+        :return: A Command instance with parsed arguments that is ready for :meth:`.__call__` or :meth:`.main`
         """
         from .parser import CommandParser
 
@@ -78,9 +77,11 @@ class Command(ABC, metaclass=CommandMeta):
         cmd_cls = cls
         with ExitStack() as stack:
             stack.enter_context(ctx)
-            while sub_cmd := CommandParser.parse_args():
+            sub_cmd = CommandParser.parse_args()
+            while sub_cmd:
                 cmd_cls = sub_cmd
                 ctx = stack.enter_context(ctx._sub_context(cmd_cls))
+                sub_cmd = CommandParser.parse_args()
 
             return cmd_cls()
 
@@ -100,16 +101,23 @@ class Command(ABC, metaclass=CommandMeta):
         """
         with self.__ctx as ctx, ctx.get_error_handler():
             self._before_main_(*args, **kwargs)
-            self.main(*args, **kwargs)
-            self._after_main_(*args, **kwargs)
+            try:
+                self.main(*args, **kwargs)
+            except BaseException:
+                if ctx.always_run_after_main:
+                    log.debug('Caught exception - running _after_main_ before propagating', exc_info=True)
+                    self._after_main_(*args, **kwargs)
+                raise
+            else:
+                self._after_main_(*args, **kwargs)
 
         return ctx.actions_taken
 
     def _before_main_(self, *args, **kwargs):
         """
-        Called by :meth:`.run` before :meth:`.main` is called.  Validates the number of ActionFlags that were specified,
-        and calls all of the specified :obj:`~.parameters.before_main` / :obj:`~.parameters.action_flag` actions
-        that were defined with ``before_main=True`` in their configured order.
+        Called by :meth:`.__call__` before :meth:`.main` is called.  Validates the number of ActionFlags that were
+        specified, and calls all of the specified :obj:`~.parameters.before_main` / :obj:`~.parameters.action_flag`
+        actions that were defined with ``before_main=True`` in their configured order.
 
         :param args: Positional arguments to pass to the :obj:`~.parameters.action_flag` methods
         :param kwargs: Keyword arguments to pass to the :obj:`~.parameters.action_flag` methods
@@ -151,7 +159,7 @@ class Command(ABC, metaclass=CommandMeta):
 
     def _after_main_(self, *args, **kwargs):
         """
-        Called by :meth:`.run` after :meth:`.main` is called.  Calls all of the specified
+        Called by :meth:`.__call__` after :meth:`.main` is called.  Calls all of the specified
         :obj:`~.parameters.after_main` / :obj:`~.parameters.action_flag` actions that were defined with
         ``before_main=False`` in their configured order.
 
