@@ -5,7 +5,7 @@ Configuration options for Command behavior.
 """
 
 from dataclasses import dataclass, fields
-from typing import TYPE_CHECKING, Optional, Any
+from typing import TYPE_CHECKING, Optional, Any, Dict, FrozenSet
 
 from .utils import Bool, _NotSet, cached_class_property
 
@@ -26,7 +26,7 @@ class CommandConfig:
     #: Whether the --help / -h action_flag should be added
     add_help: Bool = True
 
-    #: The :class:`~.error_handling.ErrorHandler` to be used by :meth:`~Command.run`
+    #: The :class:`~.error_handling.ErrorHandler` to be used by :meth:`~Command.__call__`
     error_handler: Optional['ErrorHandler'] = _NotSet
 
     #: Whether unknown arguments should be ignored (default: raise an exception when unknown arguments are encountered)
@@ -38,6 +38,10 @@ class CommandConfig:
     #: Whether missing required arguments should be allowed (default: raise an exception when they are missing)
     allow_missing: Bool = False
 
+    #: Whether :meth:`Command._after_main_` should always be called, even if an exception was raised in
+    #: :meth:`Command.main` (similar to a ``finally`` block)
+    always_run_after_main: Bool = False
+
     # #: Whether handling of dashes (``-``) and underscores (``_``) in the middle of option names should be strict
     # #: (``True``) when processing user input, or if they should be allowed to be interchanged (``False``)
     # strict_option_punctuation: Bool = False
@@ -53,10 +57,10 @@ class CommandConfig:
     # strict_sub_command_punctuation: Bool = False
 
     @cached_class_property
-    def _field_names(cls) -> frozenset[str]:  # noqa
+    def _field_names(cls) -> FrozenSet[str]:  # noqa
         return frozenset(field.name for field in fields(cls))
 
-    def as_dict(self) -> dict[str, Any]:
+    def as_dict(self) -> Dict[str, Any]:
         """
         Return a dict representing the configured options.
 
