@@ -6,8 +6,12 @@ import sys
 from collections import defaultdict
 from contextlib import AbstractContextManager
 from contextvars import ContextVar
-from functools import cached_property
-from typing import TYPE_CHECKING, Any, Union, Sequence, Optional, Iterator, Collection, cast
+from typing import TYPE_CHECKING, Any, Union, Sequence, Optional, Iterator, Collection, cast, Dict, Tuple, List
+
+try:
+    from functools import cached_property
+except ImportError:
+    from .compat import cached_property
 
 from .error_handling import ErrorHandler, NullErrorHandler, extended_error_handler
 from .exceptions import NoActiveContext
@@ -34,7 +38,8 @@ class ConfigOption:
         try:
             return ctx.__dict__[self.name]
         except KeyError:
-            if parent := ctx.parent:
+            parent = ctx.parent
+            if parent:
                 option = ctx_cls.__dict__[self.name]  # type: ConfigOption
                 return option.get_value(parent, ctx_cls)
             raise
@@ -74,6 +79,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
     allow_missing = ConfigOption()
     multiple_action_flags = ConfigOption()
     action_after_action_flags = ConfigOption()
+    always_run_after_main = ConfigOption()
     # strict_option_punctuation = ConfigOption()
     # strict_action_punctuation = ConfigOption()
     # strict_sub_command_punctuation = ConfigOption()
@@ -90,6 +96,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
         allow_missing: Bool = None,
         multiple_action_flags: Bool = None,
         action_after_action_flags: Bool = None,
+        always_run_after_main: Bool = None,
         # strict_option_punctuation: Bool = None,
         # strict_action_punctuation: Bool = None,
         # strict_sub_command_punctuation: Bool = None,
@@ -114,6 +121,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
         self.allow_missing = allow_missing
         self.multiple_action_flags = multiple_action_flags
         self.action_after_action_flags = action_after_action_flags
+        self.always_run_after_main = always_run_after_main
         # self.strict_option_punctuation = strict_option_punctuation
         # self.strict_action_punctuation = strict_action_punctuation
         # self.strict_sub_command_punctuation = strict_sub_command_punctuation
@@ -146,14 +154,15 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
             argv = self.remaining
         return self.__class__(argv, command, parent=self)
 
-    def get_parsed(self, exclude: Collection['Parameter'] = (), recursive: Bool = True) -> dict[str, Any]:
+    def get_parsed(self, exclude: Collection['Parameter'] = (), recursive: Bool = True) -> Dict[str, Any]:
         with self:
-            if recursive and (parent := self.parent):
-                parsed = parent.get_parsed(exclude, recursive)
+            if recursive and self.parent:
+                parsed = self.parent.get_parsed(exclude, recursive)
             else:
                 parsed = {}
 
-            if params := self.params:
+            params = self.params
+            if params:
                 for group in (params.positionals, params.options, (params.pass_thru,)):
                     for param in group:
                         if param and param not in exclude:
@@ -193,16 +202,17 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
         return self._provided[param]
 
     @cached_property
-    def parsed_action_flags(self) -> tuple[int, list['ActionFlag'], list['ActionFlag']]:
+    def parsed_action_flags(self) -> Tuple[int, List['ActionFlag'], List['ActionFlag']]:
         parsing = self._parsing
         try:
             action_flags = sorted(p for p in self.params.action_flags if p in parsing)
         except AttributeError:  # self.command is None
             return 0, [], []
 
-        if (num_flags := len(action_flags)) == 0:
+        if not action_flags:
             return 0, [], []
 
+        num_flags = len(action_flags)
         for i, flag in enumerate(action_flags):
             if not flag.before_main:
                 return num_flags, action_flags[:i], action_flags[i:]
