@@ -4,12 +4,17 @@
 
 import re
 import sys
-from collections.abc import Collection, Iterable
+from collections.abc import Collection, Iterable, Callable
 from inspect import stack, getsourcefile, isclass
 from pathlib import Path
-from typing import Any, Union, Optional, Type, Callable, get_type_hints, get_origin, get_args
+from typing import Any, Union, Optional, Type, get_type_hints
 from string import whitespace, printable
 
+try:
+    from typing import get_origin, get_args as _get_args
+except ImportError:  # Added in 3.8; the versions from 3.8 are copied here
+    from .compat import get_origin, _get_args
+
 try:
     from types import NoneType
 except ImportError:  # Added in 3.10
@@ -18,7 +23,6 @@ except ImportError:  # Added in 3.10
 from .exceptions import ParameterDefinitionError
 
 Bool = Union[bool, Any]
-
 _NotSet = object()
 
 
@@ -42,7 +46,9 @@ def validate_positional(
 ):
     if not value or value.startswith('-'):
         raise exc(f"Invalid {param_cls} {prefix}={value!r} - may not be empty or start with '-'")
-    elif bad := {c for c in value if (c in whitespace and c != ' ') or c not in printable}:
+
+    bad = {c for c in value if (c in whitespace and c != ' ') or c not in printable}
+    if bad:
         raise exc(f'Invalid {param_cls} {prefix}={value!r} - invalid characters: {bad}')
 
 
@@ -115,11 +121,14 @@ class ProgramMetadata:
         parts = [self.epilog] if self.epilog else []
         if parts and not extended:
             return parts[0]
-        if version := self.version:
+
+        version = self.version
+        if version:
             version = f' [ver. {version}]'
-        if email := self.email:
-            parts.append(f'Report {self.prog}{version} bugs to {email}')
-        if url := self.docs_url or self.url:
+        if self.email:
+            parts.append(f'Report {self.prog}{version} bugs to {self.email}')
+        url = self.docs_url or self.url
+        if url:
             parts.append(f'Online documentation: {url}')
         return '\n\n'.join(parts)
 
@@ -128,14 +137,25 @@ def camel_to_snake_case(text: str, delim: str = '_') -> str:
     return ''.join(f'{delim}{c}' if i and c.isupper() else c for i, c in enumerate(text)).lower()
 
 
-def get_descriptor_value_type(command_cls, attr: str):
+def get_descriptor_value_type(command_cls: type, attr: str) -> Optional[type]:
     try:
         annotation = get_type_hints(command_cls)[attr]
     except KeyError:
         return None
 
-    # TODO: Also return container type, if specified?
-    if (origin := get_origin(annotation)) is None and isinstance(annotation, type):
+    return get_annotation_value_type(annotation)
+
+
+def get_annotation_value_type(annotation) -> Optional[type]:
+    origin = get_origin(annotation)
+    """
+    Note on get_origin return values:
+    get_origin(List[str]) -> list
+    get_origin(List) -> list
+    get_origin(list[str]) -> list
+    get_origin(list) -> None
+    """
+    if origin is None and isinstance(annotation, type):
         return annotation
     elif isclass(origin) and issubclass(origin, (Collection, Iterable)):
         return _type_from_collection(origin, annotation)
@@ -144,7 +164,16 @@ def get_descriptor_value_type(command_cls, attr: str):
     return None
 
 
-def _type_from_union(annotation):
+def get_args(annotation) -> tuple:
+    """
+    Wrapper around :func:`python:typing.get_args` for 3.7~8 compatibility, to make it behave more like it does in 3.9+
+    """
+    if getattr(annotation, '_special', False):  # 3.7-3.8 generic collection alias with no content types
+        return ()
+    return _get_args(annotation)
+
+
+def _type_from_union(annotation) -> Optional[type]:
     args = get_args(annotation)
     # Note: Unions of a single argument return the argument; i.e., Union[T] returns T, so the len can never be 1
     if len(args) == 2 and NoneType in args:
@@ -152,7 +181,8 @@ def _type_from_union(annotation):
     else:
         return None
 
-    if (origin := get_origin(arg)) is None and isinstance(arg, type):
+    origin = get_origin(arg)
+    if origin is None and isinstance(arg, type):
         return arg
     elif isclass(origin) and issubclass(origin, (Collection, Iterable)):
         return _type_from_collection(origin, arg)
@@ -160,13 +190,19 @@ def _type_from_union(annotation):
         return None
 
 
-def _type_from_collection(origin, annotation):
+def _type_from_collection(origin, annotation) -> Optional[type]:
     args = get_args(annotation)
-    if not (len(args) == 2 and origin is tuple and args[1] is Ellipsis) and len(args) != 1:
+    try:
+        annotation = args[0]
+    except IndexError:  # The annotation was a collection with no content types specified
+        return origin
+
+    n_args = len(args)
+    if n_args > 2 or (n_args > 1 and (origin is not tuple or args[1] is not Ellipsis)):
         return None
 
-    annotation = args[0]
-    if (origin := get_origin(annotation)) is None and isinstance(annotation, type):
+    origin = get_origin(annotation)
+    if origin is None and isinstance(annotation, type):
         return annotation
     elif origin is Union:
         return _type_from_union(annotation)
